@@ -11,9 +11,9 @@ from .distob import (scatter, gather, vectorize, apply, call_all,
 
 engine = None
 
-__version__ = '0.1.4'
+__version__ = '0.1.5'
 
 if _have_numpy:
     from .arrays import (RemoteArray, DistArray, transpose, rollaxis,
                          expand_dims, concatenate, vstack, hstack, dstack, 
-                         split, vsplit, hsplit, dsplit)
+                         split, vsplit, hsplit, dsplit, broadcast_arrays)
